<?php
/**
 * @author Jess Mann <jess@jess-mann.com>
 * @version 1.1
 * @since 1.1
 */
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class Emergency_Call
{
	public $message = null;
	public $excerpt = null;
	public $date = null;
	public $title = null;
	public $call_number = null;
	public $incident_number = null;
	public $report = null;
	public $times;
	public $tags;
	public $category = null;

	function __construct( $message = null, $date = null, $number = null )
	{
		$this->category = '';
		$this->times = array(
			'dispatched' => '',
			'acknowledged' => '',
			'enroute' => '',
			'onscene' => '',
			'clear' => '',
			'inservice' => '',
		);
		$this->tags = array();

		if (!is_null($date))
		{
			$this->date = new DateTime($date);
		}

		if (!is_null($message))
			$this->read($message);
		if (!is_null($number))
			$this->call_number = $number;
	}

	static function find_recent_calls( $write = false )
	{
		$hostname = '{mail.example.com:993/imap/ssl/novalidate-cert}INBOX';
		$username = 'example@domain.com';
		$password = 'password';
		$from = 'messaging@iamresponding.com';
		
		/** Get the last call time. Due to timezone variances between servers, get all emails from a day before the "last post time" */
		$most_recent = get_posts(array(
			'post_type' 	=> 'ems_call',
			'numberposts' 	=> 1,
		));
		$last_post = new DateTime($most_recent[0]->post_date);
		$last_post->modify('-1 day');
		
		$call_number = get_post_meta($most_recent[0]->ID, 'call_number', true);
		
		/* try to connect */
		$inbox = imap_open($hostname,$username,$password) or die('Cannot connect to Gmail: ' . imap_last_error());
		
		/* grab emails */
		//$emails = imap_search($inbox,'SUBJECT "214" FROM "'.$from.'" SINCE "'.$date.'"');
		$emails = imap_search($inbox,'SUBJECT "214" FROM "'.$from.'" UNSEEN SINCE "'.$last_post->format('j F Y').'"');

		?>
		
		<h3>Importing calls since <?php echo $last_post->format('j F Y'); ?></h3>
		
		<?php
		/* if emails are returned, cycle through each... */
		$count = 0;
		if ($emails) 
		{
			/* begin output var */
			$output = '';
			
			/* put the newest emails on top */
			//rsort($emails);
			
			/* for every email... */
			foreach($emails as $email_number) 
			{
				/* get information specific to this email */
				$overview = imap_fetch_overview($inbox,$email_number,0);
				//TODO: Use imap_fetchstructure to determine part #. Gmail, message section is 2. Bluehost, it is 1.
				$message = imap_fetchbody($inbox,$email_number,1);

				if (empty($message))
				{
					echo 'Warning: Skipping email with empty message: "'.$overview[0]->subject.'" on '.$overview[0]->date;
					continue;
				}

				$count++;
				echo '<div class="call" style="border: 1px solid black; background:#fff; margin: 10px 0; padding: 30px;">';

				$call = new Emergency_Call($message,$overview[0]->date);
		
				if (!empty($call_number))
				{
					$call_number = $call->generate_call_number($call_number);
				}
				$id = 0;
				if ($write)
				{
					$id = $call->write();
					echo 'ID: '.$id.'<br />';
				}
		
				echo $call;
				echo '</div>';
			}
		} 
		
		/* close the connection */
		imap_close($inbox);

		if ($write)
			echo '<div>'.$count.' calls imported.</div>';
		else
			echo '<div>'.$count.' calls found. None imported.</div>';
	}
	

	function __toString()
	{
		$output = '';

		if (!is_null($this->title))
			$output .= '<h3>'.$this->title.'</h3> ';
		if (!is_null($this->date))
			$output .= $this->date->format('j F Y H:i').' ';
		if (!is_null($this->call_number))
			$output .= '<br />Call #'.$this->call_number.' ';
		if (!is_null($this->report))
			$output .= '<br />'.$this->report.' ';
		if (!is_null($this->category))
			$output .= '<br />Category: '.$this->category;
		if (!empty($this->tags))
			$output .= '<br />Tags: '.implode(',',$this->tags);

		return $output;
	}

	function error( $message )
	{
		echo $message;
		die();
	}

	function find_tags( $tags, $search, $terms )
	{
		global $wpdb;

		$search = $this->title.' '.$this->report.' '.$this->date->format('F');
		//TODO: Put in database

		$results = $wpdb->get_results( 'SELECT * FROM call_tags' );

		foreach ($results as $tag_search)
		{
			if (preg_match('/\b(?<tag>'.$tag_search->match.')\b/i',$search,$matches) > 0)
			{
				if (stripos(',',$tag_search->tags) > 0)
				{
					$tags = explode(',',$tag_search->tags);
					foreach ($tags as $tag)
						$this->add_tag($tag);
				}
				else
				{
					$this->add_tag($tag_search->tags);
				}
			}
		}

		//TODO: Date tags

		return $this->tags;
	}

	function add_tag( $tag )
	{
		if (!is_null($tag) && strlen($tag) > 0)
			$this->tags[] = strtolower(trim($tag));
		return $this->tags;
	}

	function get_tags()
	{
		return implode(',',$this->tags);
	}

	function get_category_id()
	{
		if (is_null($this->category))
			return '';
		return get_cat_ID($this->category);
	}

	function setReport( $report )
	{
		$this->report = wp_strip_all_tags($report);
		$this->report = str_replace(PHP_EOL,'',$this->report);
		return $this->report;
	}

	function setMessage( $message )
	{
		$this->message = wp_strip_all_tags($message);
		return $this->message;
	}

	function setExcerpt( $message )
	{
		$this->excerpt = wp_strip_all_tags($message);
		return $this->excerpt;
	}

	function generate_call_number( $last_call_number )
	{
		$this->call_number = self::next_call_number($last_call_number);
		return $this->call_number;
	}

	/**
	 * Generates the next call number based on the last call. Returns a call number in the format yy-ddd, 
	 * where yy is the two-digit year, and ddd is the number of the call starting at 001
	 *
	 * On error, returns false
	 * 
	 * @param string $last the last call number, in the format yy-ddd
	 * @param int $year The two-digit year of the next call. If null, we use the current year NOW
	 * @return string
	 */
	static function next_call_number( $last, $year = null )
	{
		if ($year == null)
		{
			/** Get the current year (2-digit) */
			$now = new DateTime('NOW');
			$year = $now->format('y');
		}
	
		/** Process the last call number */
		preg_match('/(?<year>\d{2})-(?<number>\d{3})/',$last,$matches);
	
		if (!isset($matches['year']) || !isset($matches['number']))
			return false;
	
		/** If a new year, then start over at call 1. Else, incriment the last call number */
		if ($year != $matches['year'])
			$call_number = $year.'-001';
		else
			$call_number = $year.'-'.str_pad( ($matches['number'] + 1), 3, '0', STR_PAD_LEFT);
	
		return $call_number;
	}

	/**
	 * Creates a post based on this 911 call
	 * @return the post id on success, 0 on failure
	 */
	function write( )
	{
		$post = array(
			'post_content'   => $this->message,
		  	'post_title'     => $this->title,
		  	'post_status'    => 'publish',
		  	'post_type'      => 'ems_call',
		  	'post_author'    => 44,
			'post_excerpt'   => $this->excerpt,
		  	'post_date'      => $this->date->format('Y-m-d H:i:s'),
			'post_category'  => array($this->get_category_id()),
		  	'tags_input'     => $this->get_tags(),
		);  

		$id = wp_insert_post($post);
		if ($id == 0)
		{
			$this->error('An error occurred inserting this email: "'.$this->title.'" @ '.$this->date->format('Y-m-d H:i:s'));
			return 0;
		}
		update_post_meta($id, 'full_description', $this->report);
		update_post_meta($id, 'call_received', $this->date->format('ymd'));
		update_post_meta($id, 'dispatched', $this->date->format('H:i'));
		if (!is_null($this->call_number))
			update_post_meta($id, 'call_number', $this->call_number);

		return $id;
	}
	
	function read( $message )
	{
		$report = $this->setReport($message);
	
		//EMS - EMS CALL CHESTNUT HILLS APARTMENTS 108 NORTHAMPTON RD AMSTERDAM  12/18/2014 1221 MALE CHEST PAIN 42.9563 -74.1864
		//TODO: Address and GPS Coordinates custom fields (for mapping later)
	
		$location = '';
		if (stripos($report,'Amsterdam') !== FALSE)
			$location = ' in town of Amsterdam';
		else if (stripos($report,'Hagaman') !== FALSE)
			$location = ' in Hagaman';
		else if (stripos($report,'Fort Johnson') !== FALSE)
			$location = ' in Fort Johnson';
		else if (stripos($report,'Tribes Hill') !== FALSE)
			$location = ' in Tribes Hill';
		else
		{
		}
	
		$type = 'a call';
		if (stripos($report,'EMS CALL') !== FALSE || stripos($report,' EMS ') !== FALSE)
			$type = 'an EMS call';
		else if (stripos($report,'wires down') !== FALSE)
			$type = 'wires down';
		else if (stripos($report,'structure fire') !== FALSE)
			$type = 'a structure fire';
		else if (stripos($report,'car fire') !== FALSE)
			$type = 'a car fire';
		else if (stripos($report,'MVA') !== FALSE || stripos($report,'Motor vehicle accident') !== FALSE)
			$type = 'a motor vehicle accident';
		else if (stripos($report,'alarm off') !== FALSE || stripos($report,'alarm-off') !== FALSE)
			$type = 'a fire alarm activation';
		else if (stripos($report,'co alarm') !== FALSE)
			$type = 'a carbon monoxide alarm activation';
		else if (stripos($report,'propane') !== FALSE)
			$type = 'a possible propane leak';
		else if (stripos($report,'fire') !== FALSE)
			$type = 'a possible fire';
		else if (stripos($report,'disabled vehicle') !== FALSE)
			$type = 'a disabled vehicle';
		else if (stripos($report,'flooding') !== FALSE)
			$type = 'possible flooding';
		else if (stripos($report,'well being') !== FALSE || stripos($report,'disorderly') !== FALSE || stripos($report,'intoxicat') !== FALSE)
			$type = 'an EMS call';
		else 
		{
			$this->error('WARNING: Unknown type: '.$report);
		}
		$type_short = preg_replace('/^an? /','',$type);
	
		$on = '';
		if (stripos($report,'route 30') !== FALSE || stripos($report,'rte 30') !== FALSE || stripos($report,'rt 30') !== FALSE || stripos($report,'rte30') !== FALSE)
			$on = ' on route 30';
		else if (stripos($report,'route 67') !== FALSE || stripos($report,'rte 67') !== FALSE || stripos($report,'rt 67') !== FALSE || stripos($report,'rte67') !== FALSE)
			$on = ' on route 67';
		else if (stripos($report,'highway 5') !== FALSE || stripos($report,'hwy 5') !== FALSE || stripos($report,'rt 5') !== FALSE || stripos($report,'route 5') !== FALSE)
			$on = ' on state highway 5';
		else if (stripos($report,'darlene drive') !== FALSE || stripos($report,'darlene dr') !== FALSE)
			$on = ' at Darlene Drive';
		else if (stripos($report,'Target') !== FALSE)
			$on = ' at Target';
		else if (stripos($report,'Walmart') !== FALSE || stripos($report,'Wallmart') !== FALSE)
			$on = ' at Walmart';
		else 
		{
			preg_match('/\d+ (?<Address>([A-Z]*){1,3} (RD|ROAD|DR|DRIVE|BLVD|ST|STREET|AVE|AVENUE|EXT|EXTENSION|TRL|TRAIL|JUNCTION))/',$report,$matches);
			if (isset($matches['Address']) && !empty($matches['Address']))
			{
				$on = ' on '.strtolower($matches['Address']);
			}
		}
	
		$this->setMessage('Our department was dispatched for '.$type.$on.$location.'.');
		$this->setExcerpt('Dispatched for '.$type.$on.$location);
		$this->title = $type_short.$on.$location;
	
		/**
		echo 'TODO: Get Date'; die();
		//Date: 21 Dec 2014 10:50:08 -0800
		preg_match('/Date: (?<Date>(Mon|Tue|Wed|Thu|Fri|Sat|Sun)?,? ?(?<Day>\d{1,2}) (?<Month>[A-Z][a-z]{2}) (?<Year>20\d{2}) (?<Time>\d{1,2}:\d{2}):\d{2} .*)/',$body,$matches);
		if (!isset($matches['Date']) || empty($matches['Date']))
		{
			echo 'Date not found: '.$report;
			die();
		}
		$date = new DateTime($matches['Date']);
		$this->date = $date->format('ymd');
		$this->time['dispatched'] = $matches['Time'];
		 */
	
		$this->find_tags();
		$this->find_category();
	}
	
	function find_category()
	{
//		echo 'TODO: Change to preg_match'; die();

		if (stripos($this->title,'Stand-By') !== FALSE || stripos($this->title,'standby') !== FALSE || stripos($this->title,'Stand By') !== FALSE)
			$this->category = 'Standby';
		else if (stripos($this->title,'hazmat') !== FALSE || stripos($this->title,'propane leak') !== FALSE || stripos($this->title,'smell of gas') !== FALSE || stripos($this->title,'gas smell') !== FALSE)
			$this->category = 'Hazmat';
		else if (stripos($this->title,'MVA') !== FALSE || stripos($this->title,'Plane Crash') !== FALSE || stripos($this->report,'Motor Vehicle Accident') !== FALSE)
			$this->category = 'MVA';
		else if (stripos($this->title,'car fire') !== FALSE || stripos($this->title,'fuel leak') !== FALSE || stripos($this->title,'vehicle fire') !== FALSE || stripos($this->title,'truck fire') !== FALSE)
			$this->category = 'Car Fire';
		else if (stripos($this->title,'CO Alarm') !== FALSE || stripos($this->title,'Carbon Monoxide') !== FALSE || stripos($this->title,'CO Detector') !== FALSE)
			$this->category = 'CO Alarm';
		else if (stripos($this->title,'Brush Fire') !== FALSE || stripos($this->title,'Grass Fire') !== FALSE || stripos($this->title,'Forest Fire') !== FALSE)
			$this->category = 'Brush Fire';
		else if (stripos($this->title,'EMS') !== FALSE)
			$this->category = 'EMS';
		else if (stripos($this->title,'Wires Down') !== FALSE || stripos($this->title,'Power Line') !== FALSE || stripos($this->title,'Power Line') !== FALSE || stripos($this->title,'telephone pole') !== FALSE || stripos($this->title,'pole down') !== FALSE || stripos($this->title,'pole leaning') !== FALSE)
			$this->category = 'Wires Down';
		else if (stripos($this->title,'Tree Down') !== FALSE)
			$this->category = 'Fire Police';
		else if (stripos($this->title,'Flood') !== FALSE)
			$this->category = 'Flooding';
		else if (stripos($this->title,'Service Call') !== FALSE)
			$this->category = 'Service Call';
		else if (stripos($this->title,'Person stuck') !== FALSE)
			$this->category = 'Entrapment';
		else if (stripos($this->title,'fire') !== FALSE || stripos($this->title,'Alarm Off') !== FALSE || stripos($this->report,'outside burn') !== FALSE || stripos($this->title,'smoke') !== FALSE || stripos($this->title,'burning') !== FALSE)
			$this->category = 'Structure Fire'; 
		else if (stripos($this->title,'Explosion') !== FALSE)
			$this->category = 'Explosion';
		else if (stripos($this->report,'dispatched to stand-by') !== FALSE || stripos($this->report,'dispatched 214 to standby') !== FALSE || stripos($this->report,'dispatched to standby') !== FALSE || stripos($this->report,'dispatched 214 to stand by') !== FALSE)
			$this->category = 'Standby';
		else if (stripos($this->report,'fire alarm') !== FALSE || stripos($this->report,'stove fire') !== FALSE || stripos($this->report,'kitchen fire') !== FALSE || stripos($this->report,'outside burn') !== FALSE || stripos($this->report,'structure fire') !== FALSE)
			$this->category = 'Structure Fire';
		else if (stripos($this->report,'car fire') !== FALSE || stripos($this->report,'fuel leak') !== FALSE || stripos($this->report,'gas leak') !== FALSE || stripos($this->report,'leaking gas') !== FALSE)
			$this->category = 'Car Fire';
		else if (stripos($this->report,'mva') !== FALSE || stripos($this->report,'motor vehicle accident') || stripos($this->report,'car accident') !== FALSE || stripos($this->report,'car rollover') !== FALSE)
			$this->category = 'MVA';
		else if (stripos($this->report,'ems') !== FALSE || stripos($this->report,'gavac') !== FALSE)
			$this->category = 'EMS';
		else if (stripos($this->report,'bomb threat') !== FALSE || stripos($this->report,'terroris') !== FALSE)
			$this->category = 'Terrorism';
		else if (stripos($this->report,'hazmat') !== FALSE || stripos($this->report,'chemical spill') !== FALSE || stripos($this->report,'propane leak') !== FALSE || stripos($this->report,'smell of gas') !== FALSE || stripos($this->report,'gas smell') !== FALSE)
			$this->category = 'Hazmat';
		else if (stripos($this->report,'Wires Down') !== FALSE)
			$this->category = 'Wires Down';
		else if (stripos($this->title,'Pump Out') !== FALSE)
			$this->category = 'Pump Out';
		else if (stripos($this->report,'Brush Fire') !== FALSE)
			$this->category = 'Brush Fire';
		else if (stripos($this->report,'fire police') !== FALSE || stripos($this->report,'shut down') !== FALSE || stripos($this->report,'Tree Down') !== FALSE)
			$this->category = 'fire police';
		else if (stripos($this->report,'Flood') !== FALSE)
			$this->category = 'Flooding';
		else if (stripos($this->title,'Mutual Aid') !== FALSE)
			$this->category = 'Mutual Aid';
		else if (stripos($this->report,'Disabled vehicle') !== FALSE)
			$this->category = 'Disabled Vehicle';
		else if (stripos($this->title,'Incorrect call') !== FALSE)
			$this->category = 'Other';
		else if (stripos($this->report,'Well Being') !== FALSE || stripos($this->report,"Disorderly") !== FALSE || stripos($this->report,"Intoxicat") !== FALSE || stripos($this->report,"Overdose") !== FALSE)
			$this->category = 'EMS';
		else {
			$this->category = 'Unknown';
			$this->error('WARNING: Unknown call type: '.$message."\n\r");
			$this->error($this->title."\n\r");
		}
	}
}
